@@ -1,4 +1,5 @@
-// +build gofuzz
+//go:build gofuzz
+
 package actionlint_fuzz
 
 import "github.com/rhysd/actionlint"
@@ -23,7 +24,7 @@ func FuzzCheck(data []byte) int {
 		actionlint.NewRuleMatrix(),
 		actionlint.NewRuleCredentials(),
 		actionlint.NewRuleShellName(),
-		actionlint.NewRuleRunnerLabel([]string{}),
+		actionlint.NewRuleRunnerLabel(),
 		actionlint.NewRuleEvents(),
 		actionlint.NewRuleGlob(),
 		actionlint.NewRuleJobNeeds(),
@@ -31,6 +32,10 @@ func FuzzCheck(data []byte) int {
 		actionlint.NewRuleEnvVar(),
 		actionlint.NewRuleID(),
 		actionlint.NewRuleExpression(ac, wc),
+		actionlint.NewRuleWorkflowCall("test.yaml", wc),
+		actionlint.NewRulePermissions(),
+		actionlint.NewRuleDeprecatedCommands(),
+		actionlint.NewRuleIfCond(),
 	}
 
 	v := actionlint.NewVisitor()
